rm(list=ls())
gc()

## @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## packages functions
rm(list=ls())

library("rlas")
library("rgl")
library("stringr")
library("VoxR")
library("data.table")

## @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

CodeR <- "/home/nica/Documents/Holger_tower_pov/CodeR/"
DataR <- "/home/nica/Documents/Holger_tower_pov/DataR"

## @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## pan3d function
pan3d <- function(button) {
    library("rgl")
    start <- list()
    begin <- function(x, y) {
        start$userMatrix <<- par3d("userMatrix")
        start$viewport <<- par3d("viewport")
        start$scale <<- par3d("scale")
        start$projection <<- rgl.projection()
        start$pos <<- rgl.window2user( x/start$viewport[3],
                                      1 - y/start$viewport[4], 0.5,
                                      projection = start$projection)
    }

    update <- function(x, y) {
        xlat <- (rgl.window2user( x/start$viewport[3],
                                 1 - y/start$viewport[4], 0.5,
                                 projection = start$projection) -
                 start$pos)*start$scale

        mouseMatrix <- translationMatrix(xlat[1], xlat[2], xlat[3])
        par3d(userMatrix = start$userMatrix %*% t(mouseMatrix) )
    }

    rgl.setMouseCallbacks(button, begin, update)
    cat("Callbacks set on button", button, "of rgl device", rgl.cur(), "\n")
}

## @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
## plot data - full path
DataPaths <- list.files(DataR,
                        pattern = "22_classified.rds",
                        ## pattern = "22_classified.las",
                        full.names = T)

Data <- fst::read_fst(DataPaths, as.data.table = T)
## Data <- read.las(DataPaths)
Data <- Data[, 1:3]

## Use a sample to speed up computations
Data <- Data[sample(1:nrow(Data), nrow(Data)*0.10), ]
gc()

## Very high values of X and Y generate problems when rendering
## the 3d graphics. Center the point cloud
Scale <- function(Vector){
    ((Vector - min(Vector))/
     (max(Vector)-min(Vector)))*
        (max(Vector)-min(Vector))}

Data[, X := Scale(Data$X)]
Data[, Y := Scale(Data$Y)]


## A point at the top of the tower, SW edge 
TopTowerSW <- data.table(X = 173.4451,
                         Y = 104.955,
                         Z = 322.6828)
TopTowerSE <- data.table(X = 175.3196,
                         Y = 105.2212,
                         Z = 322.7785)

## plot3d(Data, aspect = "iso",
##        size = 3, col = "grey21")
## spheres3d(TopTowerSE, radius = 0.3, col = "yellow")
## spheres3d(TopTowerSW, radius = 0.3, col = "red")
## pan3d(3)

## @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
# Function to compute the distance between points and the line

sifFootP <- function(PointCloud, TopTowerP, thetaDeg){
  
    theta_rad <- thetaDeg * pi / 180
    d <- c(0, -1, tan(theta_rad))
    
    cloud_south_indices <- which(PointCloud$Y < TopTowerP$Y)
    cloud_south <- PointCloud[cloud_south_indices, ]
    
    cloud_matrix <- as.matrix(cloud_south)
    TopTowerP_matrix <- matrix(rep(as.matrix(TopTowerP),
                                   nrow(cloud_south)),
                               ncol=3, byrow=TRUE)
    
    r <- cloud_matrix - TopTowerP_matrix
    
    cross_product <- cbind(r[,2]*d[3] - r[,3]*d[2],
                           r[,3]*d[1] - r[,1]*d[3],
                           r[,1]*d[2] - r[,2]*d[1])
    
    distances <- sqrt(rowSums(cross_product^2)) / sqrt(sum(d^2))
    distances <- data.table(distances = distances,
                            ids = cloud_south_indices)


    PointCloud[distances$ids, CO := distances$distances]

    PointCloud[!is.na(CO),
               H := sqrt((X - TopTowerP$X)^2 +
                         (Y - TopTowerP$Y)^2 +
                         (Z - TopTowerP$Z)^2)]
    
    PointCloud[, Angle := CO/H]
    PointCloud[, Angle := (asin(Angle))*180/pi]

    SIFOpeningAngl <- 20

    plot3d(Data, aspect = "iso",
           size = 0.5, col = "grey21")

    spheres3d(TopTowerP, radius = 0.5, col = "green")
    plot3d(PointCloud[Angle < SIFOpeningAngl, ],
           aspect = "iso", size = 1.5, col = "red",
           add = T)

}

sifFootP(PointCloud = Data,
         TopTowerP = TopTowerSW,
         thetaDeg = -30)




